@@ -1,36 +1,35 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-#  Copyright (C) 2011 Yann GUIBET <yannguibet@gmail.com>
-# This program is free software; you can redistribute it
-# and/or modify it under the terms of version 3 of the
-# GNU General Public License as published by the Free
-# Software Foundation
+# Copyright (c) 2014 Yann GUIBET <yannguibet@gmail.com>.
+# All rights reserved.
 #
-# In addition, as a special exception, the author of this
-# program gives permission to link the code of its
-# release with the OpenSSL project's "OpenSSL" library (or
-# with modified versions of it that use the same license as
-# the "OpenSSL" library), and distribute the linked
-# executables. You must obey the GNU General Public
-# License in all respects for all of the code used other
-# than "OpenSSL".  If you modify this file, you may extend
-# this exception to your version of the file, but you are
-# not obligated to do so.  If you do not wish to do so,
-# delete this exception statement from your version.
 #
-# This program is distributed in the hope that it will be
-# useful, but WITHOUT ANY WARRANTY; without even the implied
-# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-# PURPOSE.  See the GNU General Public License for more
-# details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-# You should have received a copy of the GNU General Public
-# License along with this package; if not, write to the Free
-# Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA  02110-1301 USA
+#     1. Redistributions of source code must retain the above copyright
+#     notice, this list of conditions and the following disclaimer.
+#
+#     2. Redistributions in binary form must reproduce the above copyright
+#     notice, this list of conditions and the following disclaimer in
+#     the documentation and/or other materials provided with the
+#     distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS''
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
+# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
+# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-from pyelliptic.openssl import OpenSSL
+from .openssl import OpenSSL
 
 
 class Cipher:
