@@ -1,34 +1,33 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-#  Copyright (C) 2011 Yann GUIBET <yannguibet@gmail.com>
-# This program is free software; you can redistribute it
-# and/or modify it under the terms of version 3 of the
-# GNU General Public License as published by the Free
-# Software Foundation
+# Copyright (c) 2014 Yann GUIBET <yannguibet@gmail.com>.
+# All rights reserved.
 #
-# In addition, as a special exception, the author of this
-# program gives permission to link the code of its
-# release with the OpenSSL project's "OpenSSL" library (or
-# with modified versions of it that use the same license as
-# the "OpenSSL" library), and distribute the linked
-# executables. You must obey the GNU General Public
-# License in all respects for all of the code used other
-# than "OpenSSL".  If you modify this file, you may extend
-# this exception to your version of the file, but you are
-# not obligated to do so.  If you do not wish to do so,
-# delete this exception statement from your version.
 #
-# This program is distributed in the hope that it will be
-# useful, but WITHOUT ANY WARRANTY; without even the implied
-# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-# PURPOSE.  See the GNU General Public License for more
-# details.
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions are
+# met:
 #
-# You should have received a copy of the GNU General Public
-# License along with this package; if not, write to the Free
-# Software Foundation, Inc., 51 Franklin St, Fifth Floor,
-# Boston, MA  02110-1301 USA
+#     1. Redistributions of source code must retain the above copyright
+#     notice, this list of conditions and the following disclaimer.
+#
+#     2. Redistributions in binary form must reproduce the above copyright
+#     notice, this list of conditions and the following disclaimer in
+#     the documentation and/or other materials provided with the
+#     distribution.
+#
+# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS''
+# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
+# THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS
+# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+# OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
+# IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 import sys
 import ctypes
@@ -44,7 +43,8 @@ class CipherName:
         self._blocksize = blocksize
 
     def __str__(self):
-        return "Cipher : " + self._name + " | Blocksize : " + str(self._blocksize) + " | Fonction pointer : " + str(self._pointer)
+        return ("Cipher : %s | Blocksize : %s | Fonction pointer : %s" %
+                (self._name, str(self._blocksize), str(self._pointer)))
 
     def get_pointer(self):
         return self._pointer()
@@ -122,7 +122,7 @@ class _OpenSSL:
 
         self.EC_POINT_get_affine_coordinates_GFp = self._lib.EC_POINT_get_affine_coordinates_GFp
         self.EC_POINT_get_affine_coordinates_GFp.restype = ctypes.c_int
-        self.EC_POINT_get_affine_coordinates_GFp.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p]
+        self.EC_POINT_get_affine_coordinates_GFp.argtypes = 5 * [ctypes.c_void_p]
 
         self.EC_KEY_set_private_key = self._lib.EC_KEY_set_private_key
         self.EC_KEY_set_private_key.restype = ctypes.c_int
@@ -140,7 +140,7 @@ class _OpenSSL:
 
         self.EC_POINT_set_affine_coordinates_GFp = self._lib.EC_POINT_set_affine_coordinates_GFp
         self.EC_POINT_set_affine_coordinates_GFp.restype = ctypes.c_int
-        self.EC_POINT_set_affine_coordinates_GFp.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p]
+        self.EC_POINT_set_affine_coordinates_GFp.argtypes = 5 * [ctypes.c_void_p]
 
         self.EC_POINT_new = self._lib.EC_POINT_new
         self.EC_POINT_new.restype = ctypes.c_void_p
@@ -150,6 +150,16 @@ class _OpenSSL:
         self.EC_POINT_free.restype = None
         self.EC_POINT_free.argtypes = [ctypes.c_void_p]
 
+        self.BN_CTX_free = self._lib.BN_CTX_free
+        self.BN_CTX_free.restype = None
+        self.BN_CTX_free.argtypes = [ctypes.c_void_p]
+
+        self.EC_POINT_mul = self._lib.EC_POINT_mul
+        self.EC_POINT_mul.restype = ctypes.c_int
+        self.EC_POINT_mul.argtypes = [ctypes.c_void_p, ctypes.c_void_p,
+                                      ctypes.c_void_p, ctypes.c_void_p,
+                                      ctypes.c_void_p, ctypes.c_void_p]
+
         self.EC_KEY_set_private_key = self._lib.EC_KEY_set_private_key
         self.EC_KEY_set_private_key.restype = ctypes.c_int
         self.EC_KEY_set_private_key.argtypes = [ctypes.c_void_p,
@@ -159,6 +169,10 @@ class _OpenSSL:
         self._lib.ECDH_OpenSSL.restype = ctypes.c_void_p
         self._lib.ECDH_OpenSSL.argtypes = []
 
+        self.BN_CTX_new = self._lib.BN_CTX_new
+        self._lib.BN_CTX_new.restype = ctypes.c_void_p
+        self._lib.BN_CTX_new.argtypes = []
+
         self.ECDH_set_method = self._lib.ECDH_set_method
         self._lib.ECDH_set_method.restype = ctypes.c_int
         self._lib.ECDH_set_method.argtypes = [ctypes.c_void_p, ctypes.c_void_p]
@@ -166,7 +180,9 @@ class _OpenSSL:
         self.ECDH_compute_key = self._lib.ECDH_compute_key
         self.ECDH_compute_key.restype = ctypes.c_int
         self.ECDH_compute_key.argtypes = [ctypes.c_void_p,
-                                          ctypes.c_int, ctypes.c_void_p, ctypes.c_void_p]
+                                          ctypes.c_int,
+                                          ctypes.c_void_p,
+                                          ctypes.c_void_p]
 
         self.EVP_CipherInit_ex = self._lib.EVP_CipherInit_ex
         self.EVP_CipherInit_ex.restype = ctypes.c_int
@@ -241,21 +257,24 @@ class _OpenSSL:
         self.EVP_CipherUpdate = self._lib.EVP_CipherUpdate
         self.EVP_CipherUpdate.restype = ctypes.c_int
         self.EVP_CipherUpdate.argtypes = [ctypes.c_void_p,
-                                          ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_int]
+                                          ctypes.c_void_p,
+                                          ctypes.c_void_p,
+                                          ctypes.c_void_p,
+                                          ctypes.c_int]
 
         self.EVP_CipherFinal_ex = self._lib.EVP_CipherFinal_ex
         self.EVP_CipherFinal_ex.restype = ctypes.c_int
-        self.EVP_CipherFinal_ex.argtypes = [ctypes.c_void_p,
-                                            ctypes.c_void_p, ctypes.c_void_p]
+        self.EVP_CipherFinal_ex.argtypes = 3 * [ctypes.c_void_p]
 
         self.EVP_DigestInit = self._lib.EVP_DigestInit
         self.EVP_DigestInit.restype = ctypes.c_int
-        self._lib.EVP_DigestInit.argtypes = [ctypes.c_void_p, ctypes.c_void_p]
+        self._lib.EVP_DigestInit.argtypes = 2 * [ctypes.c_void_p]
 
         self.EVP_DigestUpdate = self._lib.EVP_DigestUpdate
         self.EVP_DigestUpdate.restype = ctypes.c_int
         self.EVP_DigestUpdate.argtypes = [ctypes.c_void_p,
-                                          ctypes.c_void_p, ctypes.c_int]
+                                          ctypes.c_void_p,
+                                          ctypes.c_int]
 
         self.EVP_DigestFinal = self._lib.EVP_DigestFinal
         self.EVP_DigestFinal.restype = ctypes.c_int
@@ -268,13 +287,21 @@ class _OpenSSL:
 
         self.ECDSA_sign = self._lib.ECDSA_sign
         self.ECDSA_sign.restype = ctypes.c_int
-        self.ECDSA_sign.argtypes = [ctypes.c_int, ctypes.c_void_p,
-                                    ctypes.c_int, ctypes.c_void_p, ctypes.c_void_p, ctypes.c_void_p]
+        self.ECDSA_sign.argtypes = [ctypes.c_int,
+                                    ctypes.c_void_p,
+                                    ctypes.c_int,
+                                    ctypes.c_void_p,
+                                    ctypes.c_void_p,
+                                    ctypes.c_void_p]
 
         self.ECDSA_verify = self._lib.ECDSA_verify
         self.ECDSA_verify.restype = ctypes.c_int
-        self.ECDSA_verify.argtypes = [ctypes.c_int, ctypes.c_void_p,
-                                      ctypes.c_int, ctypes.c_void_p, ctypes.c_int, ctypes.c_void_p]
+        self.ECDSA_verify.argtypes = [ctypes.c_int,
+                                      ctypes.c_void_p,
+                                      ctypes.c_int,
+                                      ctypes.c_void_p,
+                                      ctypes.c_int,
+                                      ctypes.c_void_p]
 
         self.EVP_MD_CTX_create = self._lib.EVP_MD_CTX_create
         self.EVP_MD_CTX_create.restype = ctypes.c_void_p
@@ -289,14 +316,17 @@ class _OpenSSL:
         self.EVP_MD_CTX_destroy.argtypes = [ctypes.c_void_p]
 
         self.RAND_bytes = self._lib.RAND_bytes
-        self.RAND_bytes.restype = None
+        self.RAND_bytes.restype = ctypes.c_int
         self.RAND_bytes.argtypes = [ctypes.c_void_p, ctypes.c_int]
 
-
         self.EVP_sha256 = self._lib.EVP_sha256
         self.EVP_sha256.restype = ctypes.c_void_p
         self.EVP_sha256.argtypes = []
 
+        self.i2o_ECPublicKey = self._lib.i2o_ECPublicKey
+        self.i2o_ECPublicKey.restype = ctypes.c_int
+        self.i2o_ECPublicKey.argtypes = [ctypes.c_void_p, ctypes.c_void_p]
+
         self.EVP_sha512 = self._lib.EVP_sha512
         self.EVP_sha512.restype = ctypes.c_void_p
         self.EVP_sha512.argtypes = []
@@ -304,9 +334,14 @@ class _OpenSSL:
         self.HMAC = self._lib.HMAC
         self.HMAC.restype = ctypes.c_void_p
         self.HMAC.argtypes = [ctypes.c_void_p, ctypes.c_void_p, ctypes.c_int,
-                              ctypes.c_void_p, ctypes.c_int, ctypes.c_void_p, ctypes.c_void_p]
+                              ctypes.c_void_p, ctypes.c_int,
+                              ctypes.c_void_p, ctypes.c_void_p]
 
-        self.PKCS5_PBKDF2_HMAC = self._lib.PKCS5_PBKDF2_HMAC
+        try:
+            self.PKCS5_PBKDF2_HMAC = self._lib.PKCS5_PBKDF2_HMAC
+        except:
+            # The above is not compatible with all versions of OSX.
+            self.PKCS5_PBKDF2_HMAC = self._lib.PKCS5_PBKDF2_HMAC_SHA1
         self.PKCS5_PBKDF2_HMAC.restype = ctypes.c_int
         self.PKCS5_PBKDF2_HMAC.argtypes = [ctypes.c_void_p, ctypes.c_int,
                                            ctypes.c_void_p, ctypes.c_int,
@@ -318,27 +353,54 @@ class _OpenSSL:
 
     def _set_ciphers(self):
         self.cipher_algo = {
-            'aes-128-cbc': CipherName('aes-128-cbc', self.EVP_aes_128_cbc, 16),
-            'aes-256-cbc': CipherName('aes-256-cbc', self.EVP_aes_256_cbc, 16),
-            'aes-128-cfb': CipherName('aes-128-cfb', self.EVP_aes_128_cfb128, 16),
-            'aes-256-cfb': CipherName('aes-256-cfb', self.EVP_aes_256_cfb128, 16),
-            'aes-128-ofb': CipherName('aes-128-ofb', self._lib.EVP_aes_128_ofb, 16),
-            'aes-256-ofb': CipherName('aes-256-ofb', self._lib.EVP_aes_256_ofb, 16),
-#           'aes-128-ctr': CipherName('aes-128-ctr', self._lib.EVP_aes_128_ctr, 16),
-#           'aes-256-ctr': CipherName('aes-256-ctr', self._lib.EVP_aes_256_ctr, 16),
-            'bf-cfb': CipherName('bf-cfb', self.EVP_bf_cfb64, 8),
-            'bf-cbc': CipherName('bf-cbc', self.EVP_bf_cbc, 8),
-            'rc4': CipherName('rc4', self.EVP_rc4, 128), # 128 is the initialisation size not block size
+            'aes-128-cbc': CipherName('aes-128-cbc',
+                                      self.EVP_aes_128_cbc,
+                                      16),
+            'aes-256-cbc': CipherName('aes-256-cbc',
+                                      self.EVP_aes_256_cbc,
+                                      16),
+            'aes-128-cfb': CipherName('aes-128-cfb',
+                                      self.EVP_aes_128_cfb128,
+                                      16),
+            'aes-256-cfb': CipherName('aes-256-cfb',
+                                      self.EVP_aes_256_cfb128,
+                                      16),
+            'aes-128-ofb': CipherName('aes-128-ofb',
+                                      self._lib.EVP_aes_128_ofb,
+                                      16),
+            'aes-256-ofb': CipherName('aes-256-ofb',
+                                      self._lib.EVP_aes_256_ofb,
+                                      16),
+            # 'aes-128-ctr': CipherName('aes-128-ctr',
+            #                           self._lib.EVP_aes_128_ctr,
+            #                           16),
+            # 'aes-256-ctr': CipherName('aes-256-ctr',
+            #                           self._lib.EVP_aes_256_ctr,
+            #                           16),
+            'bf-cfb': CipherName('bf-cfb',
+                                 self.EVP_bf_cfb64,
+                                 8),
+            'bf-cbc': CipherName('bf-cbc',
+                                 self.EVP_bf_cbc,
+                                 8),
+            'rc4': CipherName('rc4',
+                              self.EVP_rc4,
+                              # 128 is the initialisation size not block size
+                              128),
         }
 
         if hasattr(self, 'EVP_aes_128_ctr'):
-            self.cipher_algo['aes-128-ctr'] = CipherName('aes-128-ctr',
-                                                         self._lib.EVP_aes_128_ctr,
-                                                         16)
+            self.cipher_algo['aes-128-ctr'] = CipherName(
+                'aes-128-ctr',
+                self._lib.EVP_aes_128_ctr,
+                16
+            )
         if hasattr(self, 'EVP_aes_256_ctr'):
-            self.cipher_algo['aes-256-ctr'] = CipherName('aes-256-ctr',
-                                                         self._lib.EVP_aes_256_ctr,
-                                                         16)
+            self.cipher_algo['aes-256-ctr'] = CipherName(
+                'aes-256-ctr',
+                self._lib.EVP_aes_256_ctr,
+                16
+            )
 
     def _set_curves(self):
         self.curves = {
@@ -416,7 +478,8 @@ class _OpenSSL:
         OpenSSL random function
         """
         buffer = self.malloc(0, size)
-        self.RAND_bytes(buffer, size)
+        if self.RAND_bytes(buffer, size) != 1:
+            raise RuntimeError("OpenSSL RAND_bytes failed")
         return buffer.raw
 
     def malloc(self, data, size):
